@@ -107,16 +107,7 @@ class Vector3:
         return self / magnitude
 
 
-class Sensor:
-
-    def __init__(self, info_dict):
-        self.update(info_dict)
-
-    def update(self, info_dict):
-        pass
-
-
-class Altimeter(Sensor):
+class Altimeter(object):
 
     """Sensor determining the robot's altitude above the terrain.
 
@@ -128,16 +119,11 @@ class Altimeter(Sensor):
 
     def __init__(self, info_object_reference, info_dict):
 
-        self.altitude = None
-        self.__info_object_reference = info_object_reference
-        super().__init__(info_dict)
-
-    def update(self, info_dict):
-
         self.altitude = info_dict["altimeter"]["altitude"]
+        self.__info_object_reference = info_object_reference
 
 
-class GPS(Sensor):
+class GPS(object):
 
     """Sensor determining the robot's world position.
 
@@ -149,17 +135,13 @@ class GPS(Sensor):
 
     def __init__(self, info_object_reference, info_dict):
 
-        self.position = None
-        self.__info_object_reference = info_object_reference
-        super().__init__(info_dict)
-
-    def update(self, info_dict):
-
         position_dict = info_dict["gps"]["position"]
+
         self.position = Vector3.from_dict(position_dict)
+        self.__info_object_reference = info_object_reference
 
 
-class Gyroscope(Sensor):
+class Gyroscope(object):
 
     """Sensor determining the orientation of the robot.
 
@@ -180,32 +162,22 @@ class Gyroscope(Sensor):
 
     def __init__(self, info_object_reference, info_dict):
 
-        self.forward = None
-        self.up = None
-        self.right = None
-        self.is_upside_down = False
-        self.__info_object_reference = info_object_reference
-        super().__init__(info_dict)
-
-    def update(self, info_dict):
-
         gyroscope_dict = info_dict["gyroscope"]
 
         self.forward = Vector3.from_dict(gyroscope_dict["forward"])
-
         self.up = Vector3.from_dict(gyroscope_dict["up"])
-
         self.right = Vector3.from_dict(gyroscope_dict["right"])
 
-        if self.__info_object_reference.coordinates_are_inverted:
+        self.__info_object_reference = info_object_reference
 
+        if self.__info_object_reference.coordinates_are_inverted:
             self.up *= -1
             self.right *= -1
 
         self.is_upside_down = gyroscope_dict["isUpsideDown"]
 
 
-class Radar(Sensor):
+class Radar(object):
 
     """Sensor determining the locations of other robots.
 
@@ -220,25 +192,19 @@ class Radar(Sensor):
 
     def __init__(self, info_object_reference, info_dict):
 
-        self.pings = None
-        self.it_ping = None
-        self.__info_object_reference = info_object_reference
-        super().__init__(info_dict)
-
-    def update(self, info_dict):
-
         radar_dict = info_dict["radar"]
 
         self.pings = []
 
         for ping in radar_dict["pings"]:
-
             self.pings.append(Vector3.from_dict(ping))
 
         self.it_ping = Vector3.from_dict(radar_dict["itPing"])
 
+        self.__info_object_reference = info_object_reference
 
-class LiDAR(Sensor):
+
+class LiDAR(object):
 
     """Sensor determining the distance to obstacles in the world in the form of a spherical matrix of distances.
 
@@ -255,11 +221,7 @@ class LiDAR(Sensor):
 
     def __init__(self, info_object_reference, info_dict):
 
-        self.distance_matrix = None
         self.__info_object_reference = info_object_reference
-        super().__init__(info_dict)
-
-    def update(self, info_dict):
 
         container_array = info_dict["lidar"]["distanceMatrix"]
         """unity can't serialize 2D arrays (grr) so we have
@@ -272,15 +234,14 @@ class LiDAR(Sensor):
 
             # flips up-down and left-right
             for container in container_array[::-1]:
-
                 self.distance_matrix.append(container["array"][::-1])
         else:
 
             for container in container_array:
-
                 self.distance_matrix.append(container["array"])
 
-class RobotInfo:
+
+class RobotInfo(object):
 
     """Class containing information about the robot
 
@@ -324,23 +285,15 @@ class RobotInfo:
         self.isIt = info_dict["isIt"]
         self.gamemode = info_dict["gameMode"]
 
-    def update(self, info_dict):
-
-        self.altimeter.update(info_dict)
-        self.gps.update(info_dict)
-        self.gyroscope.update(info_dict)
-        self.lidar.update(info_dict)
-        self.radar.update(info_dict)
-
-        self.timestamp = info_dict["timestamp"]
-        self.isIt = info_dict["isIt"]
-        self.gamemode = info_dict["gameMode"]
-
 
 class RobotConnection:
 
     """Class providing an interface between python and AutonoWar robots.
 
+    Attributes
+    ----------
+    info : RobotInfo
+        This robot's current information
     """
 
     __SEND_INTERVAL = 0.01  # in seconds
@@ -358,7 +311,8 @@ class RobotConnection:
         self.__queue_path = data_dir + "EventQueue"
         self.__info_path = data_dir + "RobotState.json"
         self.__event_buffer = []  # where events that need to be sent are stored
-        self.info = None
+        self.info = None # type: RobotInfo
+        self.__info_dict = None
         self.__should_destroy = False  # should this connection end
         self.__event_buffer_lock = threading.Lock()  # lock for threads wanting to access event_buffer
         self.__info_lock = threading.Lock()
@@ -384,6 +338,9 @@ class RobotConnection:
 
         self.info.coordinates_are_inverted = True
 
+        self.info.gyroscope = Gyroscope(self.info, self.__info_dict)  # flip coordinates immediately
+        self.info.lidar = LiDAR(self.info, self.__info_dict)
+
     def set_tire_torque(self, tire_name, torque):
 
         """Sets the torque of tire `tire_name` to `torque`.
@@ -471,11 +428,11 @@ class RobotConnection:
 
                 with self.__info_lock:
 
-                    info_dict = json.load(state_file)
-                    if self.info:
-                        self.info.update(info_dict)
-                    else:
-                        self.info = RobotInfo(info_dict)
+                    tmp_dict = json.load(state_file)
+                    self.__info_dict = tmp_dict
+
+                tmp_info = RobotInfo(self.__info_dict) # to prevent a race condition
+                self.info = tmp_info
 
             except (EnvironmentError, json.JSONDecodeError):
                 continue
